local mPhotonView, mJavaBridge = ...

-- native 版本的首页光子卡片 lua 脚本，这里是 V1 版本，
-- 主要是为了防止客户端没有成功拉取到视图相关文件而做的打底，保证首页 Banner 一定会展示

local TAG = "home_page_banner"

local DEFAULT_BANNER_PAGE_NAME = "default_banner_page.xml"
local IMAGE_BANNER_PAGE_NAME = "image_banner_page.xml"
local APP_BANNER_PAGE_NAME = "app_banner_page.xml"

local SUPPORT_SUB_VIEW_NAME = {}
-- 填充支持的 sub view name
SUPPORT_SUB_VIEW_NAME[1] = IMAGE_BANNER_PAGE_NAME
SUPPORT_SUB_VIEW_NAME[2] = APP_BANNER_PAGE_NAME

-- !!! native 查找部分专用 !!!
-- native view xml name
local NATIVE_SUB_VIEW_NAME = {}
-- 填充支持的 sub view name, 对应预埋在本地的 view name,
local NATIVE_DEFAULT_BANNER_PAGE_NAME = "native_"..DEFAULT_BANNER_PAGE_NAME
local NATIVE_IMAGE_BANNER_PAGE_NAME = "native_"..IMAGE_BANNER_PAGE_NAME
local NATIVE_APP_BANNER_PAGE_NAME = "native_"..APP_BANNER_PAGE_NAME
NATIVE_SUB_VIEW_NAME[1] = NATIVE_IMAGE_BANNER_PAGE_NAME
NATIVE_SUB_VIEW_NAME[2] = NATIVE_APP_BANNER_PAGE_NAME
-- !!! native 查找 sub view 部分填充结束 !!!

local mRecyclerView
local mIndicateView
local dataList, viewList
local pageCount

local pageChangeListener
local PageChangeListener = {}
function PageChangeListener.onPageChange(beforePageIndex, afterPageIndex) 
    mIndicateView:getView():setCurrentSelectedDotIndex(afterPageIndex)
end

-- 记录卡片 Banner 的大位置
local bigPosition = ""

function initBannerHeight()
    local screenWidth = mJavaBridge:getScreenWidth()
    -- 参照 native 实现：banner 高度为宽度的 7 / 18
    local bannerHeight = mJavaBridge:px2dip((screenWidth * 7) / 18)
    Rapid:setData("banner_height", tostring(bannerHeight))
end

function init()
    initBannerHeight()
    mRecyclerView = mPhotonView:getParser():getChildView("banner")
    pageChangeListener = luajava.createProxy("com.tencent.rapidview.deobfuscated.control.IBannerPageChangeListener", PageChangeListener)
    mRecyclerView:getView():addPageChangeListener(pageChangeListener)
    mIndicateView = mPhotonView:getParser():getChildView("indicate_view")
end

function fixBigPosition()
    local oldSlot = Rapid:getData("slotid")
    local newSlot = ""
    if (oldSlot == nil or oldSlot == "") then
        oldSlot = Rapid:getData("slotId")
    end
    if (oldSlot == nil or oldSlot == "") then
        bigPosition = "001"
        return
    end
    local slotLen = string.len(oldSlot)
    local _index = string.find(oldSlot, "_")
    if (_index == nil or _index == -1 or _index > slotLen) then
        bigPosition = "001"
        return
    end
    local newSlot = string.sub(oldSlot, _index + 1)
    if (newSlot == nil or newSlot == "") then
        bigPosition = "001"
        return
    end
    -- 一般 slotId 都是只有一个 _ 字符，会走这个 if 分支
    local _secondIndex = string.find(newSlot, "_")
    if (_secondIndex == nil or _secondIndex == -1 or _secondIndex > slotLen) then
        bigPosition = newSlot
        return
    end
    -- 如果存在两个 _ 字符，截取第一个 _ 和第二个 _ 之间的数据作为大位置
    if (1 < _secondIndex) then
        bigPosition = string.sub(newSlot, 1, _secondIndex - 1)
    end
end

function fixSlotId()
    fixBigPosition()
    local newSlot = ""
    -- 原有 native banner 的 column 为 21，这里保持一致
    if (bigPosition == nil or bigPosition == "") then
        newSlot = "21_001"
    else
        newSlot = "21_"..tostring(bigPosition)
    end
    Rapid:setData("slotId", newSlot)
    Rapid:setData("slotid", newSlot)
end

function fillValue()
    fixSlotId()
    dataList = {}
    viewList = {}
    local totalSubViewNum = mPhotonView:getParser():getBinder():get("page_count")
    pageCount = tonumber(totalSubViewNum)
    local viewIndex = 0
    for i = 1, pageCount do
        local viewName = getViewName(i, getDataByIndex("image_url_", i))
        local itemData = makePageData(viewName, i)
        -- 判断得到的 Banner 页和数据是合法的，则进行添加
        if (judgeBannerPageValid(viewName, itemData)) then
            viewIndex = viewIndex + 1
            dataList[viewIndex] = itemData
            viewList[viewIndex] = viewName
        end
    end
    -- 如果最后没有任何一页 Banner 是合法的，则将 Banner 隐藏
    if (viewIndex == 0) then
        mPhotonView:getParser():getChildView("banner_view"):getView():setVisibility(8)
    else
        -- 更新过滤后的 page_count 值，给指示器使用
        Rapid:setData("page_count", viewIndex)
        mPhotonView:getParser():getChildView("banner_view"):getView():setVisibility(0)
        -- 参照原有 native 逻辑，如果最后添加的 banner 页数不大于 1 ，则不自动播放，将指示器隐藏，同时不能手动左右滑动
        if (viewIndex <= 1) then
            Rapid:setData("auto_play", "0")
            Rapid:setData("can_touch_scroll", "0")
            mIndicateView:getView():setVisibility(8)
        else
            mIndicateView:getView():setVisibility(0)
        end
    end
    mRecyclerView:getView():clear()
    mRecyclerView:getView():updateData(viewList, dataList)
end

-- 判断创建的 view name 和 view data 是否合法
function judgeBannerPageValid(viewName, viewData)
    if (viewName == "" or viewName == nil or viewData == nil) then
        return false
    end
    -- 默认的 Banner 页和图片 Banner 页和 App Banner 页一定要有图片 url 数据，不然就是空白页
    if (viewName == DEFAULT_BANNER_PAGE_NAME or viewName == IMAGE_BANNER_PAGE_NAME
            or viewName == APP_BANNER_PAGE_NAME) then
        local imageUrl = viewData["image_url"]
        local defaultImageUrl = viewData["default_image_url"]
        -- 主动设置和备用的图片都为空，则该页数据不合法，返回 false
        if ((imageUrl == nil or imageUrl == "") and (defaultImageUrl == nil or defaultImageUrl == "")) then
            return false
        elseif (imageUrl == nil or imageUrl == "") then
            viewData["image_url"] = defaultImageUrl
        end
    end
    return true
end

function getViewName(index, imageUrl)
    local viewName = getData("page_view_name_"..index)
    if (viewName == nil or viewName == "") then
        return ""
    end
    -- 检测得到的视图是否在支持的视图范围内
    for i,v in ipairs(SUPPORT_SUB_VIEW_NAME) do
        if (v == viewName) then
            -- 如果当前页为 app 页并且其数据中图片 url 值为空，则返回打底视图名
            if (v == APP_BANNER_PAGE_NAME and (imageUrl == nil or imageUrl == "")) then
                return NATIVE_DEFAULT_BANNER_PAGE_NAME
            end
            -- 版本 V1 ，判断是否是初始的图片页/应用页，如果是，换成 native 预埋的视图名
            if (v == IMAGE_BANNER_PAGE_NAME or v == APP_BANNER_PAGE_NAME) then
                return NATIVE_SUB_VIEW_NAME[i]
            end
        end
    end
    -- 不在支持的视图范围内，返回打底视图名
    return NATIVE_DEFAULT_BANNER_PAGE_NAME
end

function makePageData(pageViewName, pageIndex)
    if (pageViewName == nil or pageViewName == "") then
        return {}
    end
    local pageData = {}
    if (pageViewName == NATIVE_IMAGE_BANNER_PAGE_NAME) then
        pageData = makeImagePageData(pageIndex)
    elseif (pageViewName == NATIVE_APP_BANNER_PAGE_NAME) then
        pageData = makeAppPageData(pageIndex)
    elseif (pageViewName == NATIVE_DEFAULT_BANNER_PAGE_NAME) then
        pageData = makeDefaultPageData(pageIndex)
    end
    return pageData
end

function makeImagePageData(pageIndex)
    local pageData = {}
    if (pageIndex == nil) then
        return pageData
    end
    local imageUrl = getDataByIndex("image_url_", pageIndex)
    if (imageUrl ~= nil and imageUrl ~= "") then
        pageData["image_url"] = imageUrl
    end
    local jumpUrl = getDataByIndex("jump_url_", pageIndex)
    if (jumpUrl ~= nil and jumpUrl ~= "") then
        pageData["jump_url"] = jumpUrl
    end
    makeReportData(pageData, pageIndex)
    return pageData
end

function makeAppPageData(pageIndex)
    local pageData = makeImagePageData(pageIndex)
    local appModel = getDataByIndex("appModel_", pageIndex)
    if (appModel ~= nil) then
        pageData["app_model"] = appModel
    end
    local appId = getDataByIndex("app_id_", pageIndex)
    if (dataIsValid(appId)) then
        pageData["app_id"] = appId
    end
    local appName = getDataByIndex("app_name_", pageIndex)
    if (dataIsValid(appName)) then
        pageData["app_name"] = appName
    end
    local appIconUrl = getDataByIndex("app_icon_", pageIndex)
    if (dataIsValid(appIconUrl)) then
        pageData["app_icon_url"] = appIconUrl
    end
    local appSize = getDataByIndex("app_size_", pageIndex)
    if (dataIsValid(appSize)) then
        pageData["app_size"] = appSize
    end
    local darkMode = getDataByIndex("dispPattern_", pageIndex)
    if (dataIsValid(darkMode)) then
        pageData["dark_mode"] = darkMode
    end
    return pageData
end

function makeDefaultPageData(pageIndex)
    return makeImagePageData(pageIndex)
end

function getDataByIndex(keyPrefix, pageIndex)
    if (keyPrefix == nil or keyPrefix == "" or pageIndex == nil or pageIndex == "") then
        return ""
    end
    return getData(tostring(keyPrefix)..tostring(pageIndex))
end

function getData(key)
    if (key == nil or key == "") then
        return ""
    end
    return mPhotonView:getParser():getBinder():get(key)
end

function dataIsValid(data)
    return data ~= nil and data ~= ""
end

function makeReportData(pageData, pageIndex)
    if (pageData == nil or pageIndex == nil) then
        return
    end
    pageData["id"] = getData("id_"..pageIndex)
    pageData["sourceScene"] = getData("sourceScene")
    pageData["sourcescene"] = pageData["sourceScene"]
    pageData["sourceSceneSlotId"] = getData("sourceSceneSlotId")
    pageData["sourcesceneslotid"] = pageData["sourceSceneSlotId"]
    pageData["modelType"] = getData("modelType")
    pageData["modeltype"] = pageData["modelType"]
    pageData["scene"] = getData("scene")
    pageData["slotId"] = getData("slotId")
    pageData["slotid"] = pageData["slotId"]
    pageData["subposition"] = make3BitPosition(pageIndex)
    pageData["subPosition"] = pageData["subposition"]
    -- 根据原有 native 逻辑，如果 id 是 0/-1，这里的 slotid 槽位（column）要改为 07
    if (pageData["id"] == "0" or pageData["id"] == "-1") then
        pageData["slotId"] = "07_"..tostring(bigPosition)
    end
    pageData["recommendid"] = getData("recommend_id_"..pageIndex)
    pageData["recommendId"] = pageData["recommendid"]
    --  原来的 native 实现中 banner 需要额外上报 contentId 字段
    if (dataIsValid(pageData["id"])) then
        pageData["content_id"] = "3_"..tostring(pageData["id"])
        pageData["contentid"] = pageData["content_id"]
        pageData["contentId"] = pageData["content_id"]
    end
end

-- 获取 3 位的位置值
function make3BitPosition(pageIndex)
    if (pageIndex == nil or pageIndex == "") then
        return ""
    end
    local intPageIndex = tonumber(pageIndex)
    if (intPageIndex < 10) then
        return "00"..tostring(pageIndex)
    elseif (intPageIndex < 100) then
        return "0"..tostring(pageIndex)
    else 
        return tostring(pageIndex)
    end
end

function onPause()
    local realBannerView = mRecyclerView:getView()
    if (realBannerView ~= nil and realBannerView:isPlaying()) then
        realBannerView:stopPlay()
        realBannerView:scrollOffset()
    end
end

function onResume()
    local realBannerView = mRecyclerView:getView()
    if (realBannerView ~= nil and realBannerView:isAutoPlay() and realBannerView:isPlaying() == false) then
        realBannerView:startPlay()
    end
end